@@ -7,19 +7,44 @@ from functools import partial
 
 from sql import Null
 
-from ..model import ModelView, ModelStorage, ModelSQL, DeactivableMixin, fields
-from ..tools import file_open
-from ..pyson import PYSONDecoder, PYSON, Eval
-from ..transaction import Transaction
-from ..pool import Pool
-from ..cache import Cache
-from ..rpc import RPC
+from trytond.cache import Cache, MemoryCache
+from trytond.config import config
+from trytond.i18n import gettext
+from trytond.model import (
+    ModelView, ModelStorage, ModelSQL, DeactivableMixin, fields,
+    sequence_ordered, ModelSingleton)
+from trytond.model.exceptions import ValidationError
+from trytond.pool import Pool
+from trytond.pyson import PYSONDecoder, PYSON, Eval
+from trytond.pyson import PYSONEncoder
+from trytond.rpc import RPC
+from trytond.tools import file_open
+from trytond.transaction import Transaction
+
+if not config.get('html', 'plugins-ir.action.report-report_content_html'):
+    config.set(
+        'html', 'plugins-ir.action.report-report_content_html', 'fullpage')
+
+
+class WizardModelError(ValidationError):
+    pass
+
+
+class EmailError(ValidationError):
+    pass
+
+
+class ViewError(ValidationError):
+    pass
+
+
+class DomainError(ValidationError):
+    pass
+
+
+class ContextError(ValidationError):
+    pass
 
-__all__ = [
-    'Action', 'ActionKeyword', 'ActionReport',
-    'ActionActWindow', 'ActionActWindowView', 'ActionActWindowDomain',
-    'ActionWizard', 'ActionURL',
-    ]
 
 EMAIL_REFKEYS = set(('cc', 'to', 'subject'))
 
@@ -29,20 +54,27 @@ class Action(DeactivableMixin, ModelSQL, ModelView):
     __name__ = 'ir.action'
     name = fields.Char('Name', required=True, translate=True)
     type = fields.Char('Type', required=True, readonly=True)
+    records = fields.Selection([
+            ('selected', "Selected"),
+            ('listed', "Listed"),
+            ], "Records",
+        help="The records on which the action runs.")
     usage = fields.Char('Usage')
     keywords = fields.One2Many('ir.action.keyword', 'action',
             'Keywords')
-    groups = fields.Many2Many('ir.action-res.group', 'action', 'group',
-            'Groups')
     icon = fields.Many2One('ir.ui.icon', 'Icon')
 
     @classmethod
     def __setup__(cls):
         super(Action, cls).__setup__()
         cls.__rpc__.update({
-                'get_action_id': RPC(),
+                'get_action_value': RPC(instantiate=0, cache=dict(days=1)),
                 })
 
+    @classmethod
+    def default_records(cls):
+        return 'selected'
+
     @staticmethod
     def default_usage():
         return None
@@ -76,17 +108,36 @@ class Action(DeactivableMixin, ModelSQL, ModelView):
                     return action.action.id
 
     @classmethod
-    def get_action_values(cls, type_, action_ids):
-        Action = Pool().get(type_)
-        columns = set(Action._fields.keys())
-        columns.add('icon.rec_name')
-        to_remove = ()
+    def get_action_values(cls, type_, action_ids, columns=None):
+        pool = Pool()
+        Action = pool.get(type_)
+        if columns is None:
+            columns = []
+        columns += ['id', 'name', 'type', 'records', 'icon.rec_name']
         if type_ == 'ir.action.report':
-            to_remove = ('report_content_custom', 'report_content')
+            columns += ['report_name', 'direct_print']
         elif type_ == 'ir.action.act_window':
-            to_remove = ('domain', 'context', 'search_value')
-        columns.difference_update(to_remove)
-        return Action.read(action_ids, list(columns))
+            columns += [
+                'views', 'domains', 'res_model', 'limit',
+                'context_model', 'context_domain',
+                'pyson_domain', 'pyson_context', 'pyson_order',
+                'pyson_search_value']
+        elif type_ == 'ir.action.wizard':
+            columns += ['wiz_name', 'window']
+        elif type_ == 'ir.action.url':
+            columns += ['url']
+        actions = Action.read(action_ids, columns)
+        if type_ == 'ir.action.act_window':
+            for values in actions:
+                if (values['res_model']
+                        and issubclass(
+                            pool.get(values['res_model']), ModelSingleton)):
+                    values['res_id'] = 1
+        return actions
+
+    def get_action_value(self):
+        return self.get_action_values(
+            self.type, [self.get_action_id(self.id)])[0]
 
 
 class ActionKeyword(ModelSQL, ModelView):
@@ -102,17 +153,13 @@ class ActionKeyword(ModelSQL, ModelView):
     model = fields.Reference('Model', selection='models_get')
     action = fields.Many2One('ir.action', 'Action',
         ondelete='CASCADE', select=True)
-    groups = fields.Function(fields.One2Many('res.group', None, 'Groups'),
-        'get_groups', searcher='search_groups')
     _get_keyword_cache = Cache('ir_action_keyword.get_keyword')
 
     @classmethod
     def __setup__(cls):
         super(ActionKeyword, cls).__setup__()
-        cls.__rpc__.update({'get_keyword': RPC()})
-        cls._error_messages.update({
-                'wrong_wizard_model': ('Wrong wizard model in keyword action '
-                    '"%s".'),
+        cls.__rpc__.update({
+                'get_keyword': RPC(cache=dict(days=1)),
                 })
 
     @classmethod
@@ -122,13 +169,6 @@ class ActionKeyword(ModelSQL, ModelView):
         table = cls.__table_handler__(module_name)
         table.index_action(['keyword', 'model'], 'add')
 
-    def get_groups(self, name):
-        return [g.id for g in self.action.groups]
-
-    @classmethod
-    def search_groups(cls, name, clause):
-        return [('action.' + clause[0],) + tuple(clause[1:])]
-
     @classmethod
     def validate(cls, actions):
         super(ActionKeyword, cls).validate(actions)
@@ -147,8 +187,9 @@ class ActionKeyword(ModelSQL, ModelView):
                 if action_wizard.model:
                     if not str(self.model).startswith(
                             '%s,' % action_wizard.model):
-                        self.raise_user_error('wrong_wizard_model', (
-                                action_wizard.rec_name,))
+                        raise WizardModelError(
+                            gettext('ir.msg_action_wrong_wizard_model',
+                                name=action_wizard.rec_name))
 
     @staticmethod
     def _convert_vals(vals):
@@ -163,7 +204,7 @@ class ActionKeyword(ModelSQL, ModelView):
     def models_get():
         pool = Pool()
         Model = pool.get('ir.model')
-        return [(m.model, m.name) for m in Model.search([])]
+        return [(None, '')] + Model.get_name_items()
 
     @classmethod
     def delete(cls, keywords):
@@ -195,7 +236,9 @@ class ActionKeyword(ModelSQL, ModelView):
 
     @classmethod
     def get_keyword(cls, keyword, value):
-        Action = Pool().get('ir.action')
+        pool = Pool()
+        Action = pool.get('ir.action')
+        Menu = pool.get('ir.ui.menu')
         key = (keyword, tuple(value))
         keywords = cls._get_keyword_cache.get(key)
         if keywords is not None:
@@ -205,7 +248,10 @@ class ActionKeyword(ModelSQL, ModelView):
 
         clause = [
             ('keyword', '=', keyword),
-            ('model', '=', model + ',-1'),
+            ['OR',
+                ('model', '=', model + ',-1'),
+                ('model', '=', None),
+                ],
             ]
         if model_id >= 0:
             clause = ['OR',
@@ -225,6 +271,18 @@ class ActionKeyword(ModelSQL, ModelView):
             for value in Action.get_action_values(type_, action_ids):
                 value['keyword'] = keyword
                 keywords.append(value)
+        if keyword == 'tree_open' and model == Menu.__name__:
+            menu = Menu(model_id)
+            if menu.parent:
+                for value in keywords:
+                    if value['type'] == 'ir.action.act_window':
+                        value['name'] = menu.name
+                        parent = menu.parent
+                        if parent.name == value['name']:
+                            parent = parent.parent
+                        if parent:
+                            value['name'] = (
+                                parent.rec_name + ' / ' + value['name'])
         keywords.sort(key=itemgetter('name'))
         cls._get_keyword_cache.set(key, keywords)
         return keywords
@@ -236,7 +294,9 @@ class ActionMixin(ModelSQL):
 
     @classmethod
     def __setup__(cls):
+        pool = Pool()
         super(ActionMixin, cls).__setup__()
+        Action = pool.get('ir.action')
         for name in dir(Action):
             field = getattr(Action, name)
             if (isinstance(field, fields.Field)
@@ -256,13 +316,18 @@ class ActionMixin(ModelSQL):
 
     @classmethod
     def get_action(cls, ids, names):
+        def identical(v):
+            return v
+
+        def list_int(v):
+            return list(map(int, v))
         records = cls.browse(ids)
         result = {}
         for name in names:
             result[name] = values = {}
             for record in records:
                 value = getattr(record, 'action')
-                convert = lambda v: v
+                convert = identical
                 if value is not None:
                     value = getattr(value, name)
                     if isinstance(value, ModelStorage):
@@ -271,7 +336,7 @@ class ActionMixin(ModelSQL):
                         else:
                             convert = int
                     elif isinstance(value, (list, tuple)):
-                        convert = lambda v: [r.id for r in v]
+                        convert = list_int
                 values[record.id] = convert(value)
         return result
 
@@ -366,16 +431,10 @@ class ActionMixin(ModelSQL):
         return new_records
 
     @classmethod
-    def get_groups(cls, name, action_id=None):
-        # TODO add cache
-        domain = [
-            (cls._action_name, '=', name),
-            ]
-        if action_id:
-            domain.append(('id', '=', action_id))
-        actions = cls.search(domain)
-        groups = {g.id for a in actions for g in a.groups}
-        return groups
+    def fetch_action(cls, action_id):
+        fields = list(cls._fields.keys())
+        return cls.search_read(
+            [('action', '=', action_id)], fields_names=fields, limit=1)
 
 
 class ActionReport(ActionMixin, ModelSQL, ModelView):
@@ -397,6 +456,14 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
         'get_report_content', setter='set_report_content')
     report_content_name = fields.Function(fields.Char('Content Name'),
         'on_change_with_report_content_name')
+    report_content_html = fields.Function(fields.Binary(
+            "Content HTML",
+            states={
+                'invisible': ~Eval('template_extension').in_(
+                    ['html', 'xhtml']),
+                },
+            depends=['template_extension']),
+        'get_report_content_html', setter='set_report_content_html')
     action = fields.Many2One('ir.action', 'Action', required=True,
             ondelete='CASCADE')
     direct_print = fields.Boolean('Direct Print')
@@ -481,19 +548,9 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
             ('xpm', 'X PixMap'),
             ], translate=False,
         string='Extension', help='Leave empty for the same as template, '
-        'see LibreOffice documentation for compatible format')
+        'see LibreOffice documentation for compatible format.')
     module = fields.Char('Module', readonly=True, select=True)
-    email = fields.Char('Email',
-        help='Python dictonary where keys define "to" "cc" "subject"\n'
-        "Example: {'to': 'test@example.com', 'cc': 'user@example.com'}")
-    pyson_email = fields.Function(fields.Char('PySON Email'), 'get_pyson')
-
-    @classmethod
-    def __setup__(cls):
-        super(ActionReport, cls).__setup__()
-        cls._error_messages.update({
-                'invalid_email': 'Invalid email definition on report "%s".',
-                })
+    _template_cache = MemoryCache('ir.action.report.template', context=False)
 
     @classmethod
     def __register__(cls, module_name):
@@ -545,28 +602,6 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
     def default_module():
         return Transaction().context.get('module') or ''
 
-    @classmethod
-    def validate(cls, reports):
-        super(ActionReport, cls).validate(reports)
-        cls.check_email(reports)
-
-    @classmethod
-    def check_email(cls, reports):
-        "Check email"
-        for report in reports:
-            if report.email:
-                try:
-                    value = PYSONDecoder().decode(report.email)
-                except Exception:
-                    value = None
-                if isinstance(value, dict):
-                    inkeys = set(value)
-                    if not inkeys <= EMAIL_REFKEYS:
-                        cls.raise_user_error('invalid_email', (
-                                report.rec_name,))
-                else:
-                    cls.raise_user_error('invalid_email', (report.rec_name,))
-
     def get_is_custom(self, name):
         return bool(self.report_content_custom)
 
@@ -597,6 +632,16 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
     def set_report_content(cls, records, name, value):
         cls.write(records, {'%s_custom' % name: value})
 
+    @classmethod
+    def get_report_content_html(cls, reports, name):
+        return cls.get_report_content(reports, name[:-5])
+
+    @classmethod
+    def set_report_content_html(cls, reports, name, value):
+        if value is not None:
+            value = value.encode('utf-8')
+        cls.set_report_content(reports, name[:-5], value)
+
     @fields.depends('name', 'template_extension')
     def on_change_with_report_content_name(self, name=None):
         return ''.join(
@@ -606,9 +651,7 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
     def get_pyson(cls, reports, name):
         pysons = {}
         field = name[6:]
-        defaults = {
-            'email': '{}',
-            }
+        defaults = {}
         for report in reports:
             pysons[report.id] = (getattr(report, field)
                 or defaults.get(field, 'null'))
@@ -643,8 +686,15 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
                 args.extend((reports, values))
             reports, values = args[:2]
             args = args[2:]
+        cls._template_cache.clear()
         super(ActionReport, cls).write(reports, values, *args)
 
+    def get_template_cached(self):
+        return self._template_cache.get(self.id)
+
+    def set_template_cached(self, template):
+        self._template_cache.set(self.id, template)
+
 
 class ActionActWindow(ActionMixin, ModelSQL, ModelView):
     "Action act window"
@@ -656,18 +706,18 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
     context_model = fields.Char('Context Model')
     context_domain = fields.Char(
         "Context Domain",
-        help="Part of the domain that will be evaluated on each refresh")
+        help="Part of the domain that will be evaluated on each refresh.")
     act_window_views = fields.One2Many('ir.action.act_window.view',
             'act_window', 'Views')
     views = fields.Function(fields.Binary('Views'), 'get_views')
     act_window_domains = fields.One2Many('ir.action.act_window.domain',
         'act_window', 'Domains')
     domains = fields.Function(fields.Binary('Domains'), 'get_domains')
-    limit = fields.Integer('Limit', help='Default limit for the list view')
+    limit = fields.Integer('Limit', help='Default limit for the list view.')
     action = fields.Many2One('ir.action', 'Action', required=True,
             ondelete='CASCADE')
     search_value = fields.Char('Search Criteria',
-            help='Default search criteria for the list view')
+            help='Default search criteria for the list view.')
     pyson_domain = fields.Function(fields.Char('PySON Domain'), 'get_pyson')
     pyson_context = fields.Function(fields.Char('PySON Context'),
             'get_pyson')
@@ -678,16 +728,8 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(ActionActWindow, cls).__setup__()
-        cls._error_messages.update({
-                'invalid_views': ('Invalid view "%(view)s" for action '
-                    '"%(action)s".'),
-                'invalid_domain': ('Invalid domain or search criteria '
-                    '"%(domain)s" on action "%(action)s".'),
-                'invalid_context': ('Invalid context "%(context)s" on action '
-                    '"%(action)s".'),
-                })
         cls.__rpc__.update({
-                'get': RPC(),
+                'get': RPC(cache=dict(days=1)),
                 })
 
     @classmethod
@@ -737,28 +779,28 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
                 for act_window_view in action.act_window_views:
                     view = act_window_view.view
                     if view.model != action.res_model:
-                        cls.raise_user_error('invalid_views', {
-                                'view': view.rec_name,
-                                'action': action.rec_name,
-                                })
+                        raise ViewError(
+                            gettext('ir.msg_action_invalid_views',
+                                view=view.rec_name,
+                                action=action.rec_name))
                     if view.type == 'board':
-                        cls.raise_user_error('invalid_views', {
-                                'view': view.rec_name,
-                                'action': action.rec_name,
-                                })
+                        raise ViewError(
+                            gettext('ir.msg_action_invalid_views',
+                                view=view.rec_name,
+                                action=action.rec_name))
             else:
                 for act_window_view in action.act_window_views:
                     view = act_window_view.view
                     if view.model:
-                        cls.raise_user_error('invalid_views', {
-                                'view': view.rec_name,
-                                'action': action.rec_name,
-                                })
+                        raise ViewError(
+                            gettext('ir.msg_action_invalid_views',
+                                view=view.rec_name,
+                                action=action.rec_name))
                     if view.type != 'board':
-                        cls.raise_user_error('invalid_views', {
-                                'view': view.rec_name,
-                                'action': action.rec_name,
-                                })
+                        raise ViewError(
+                            gettext('ir.msg_action_invalid_views',
+                                view=view.rec_name,
+                                action=action.rec_name))
 
     @classmethod
     def check_domain(cls, actions):
@@ -770,29 +812,29 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
                 try:
                     value = PYSONDecoder().decode(domain)
                 except Exception:
-                    cls.raise_user_error('invalid_domain', {
-                            'domain': domain,
-                            'action': action.rec_name,
-                            })
+                    raise DomainError(
+                        gettext('ir.msg_action_invalid_domain',
+                            domain=domain,
+                            action=action.rec_name))
                 if isinstance(value, PYSON):
                     if not value.types() == set([list]):
-                        cls.raise_user_error('invalid_domain', {
-                                'domain': domain,
-                                'action': action.rec_name,
-                                })
+                        raise DomainError(
+                            gettext('ir.msg_action_invalid_domain',
+                                domain=domain,
+                                action=action.rec_name))
                 elif not isinstance(value, list):
-                    cls.raise_user_error('invalid_domain', {
-                            'domain': domain,
-                            'action': action.rec_name,
-                            })
+                    raise DomainError(
+                        gettext('ir.msg_action_invalid_domain',
+                            domain=domain,
+                            action=action.rec_name))
                 else:
                     try:
                         fields.domain_validate(value)
                     except Exception:
-                        cls.raise_user_error('invalid_domain', {
-                                'domain': domain,
-                                'action': action.rec_name,
-                                })
+                        raise DomainError(
+                            gettext('ir.msg_action_invalid_domain',
+                                domain=domain,
+                                action=action.rec_name))
 
     @classmethod
     def check_context(cls, actions):
@@ -802,29 +844,29 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
                 try:
                     value = PYSONDecoder().decode(action.context)
                 except Exception:
-                    cls.raise_user_error('invalid_context', {
-                            'context': action.context,
-                            'action': action.rec_name,
-                            })
+                    raise ContextError(
+                        gettext('ir.msg_action_invalid_context',
+                            context=action.context,
+                            action=action.rec_name))
                 if isinstance(value, PYSON):
                     if not value.types() == set([dict]):
-                        cls.raise_user_error('invalid_context', {
-                                'context': action.context,
-                                'action': action.rec_name,
-                                })
+                        raise ContextError(
+                            gettext('ir.msg_action_invalid_context',
+                                context=action.context,
+                                action=action.rec_name))
                 elif not isinstance(value, dict):
-                    cls.raise_user_error('invalid_context', {
-                            'context': action.context,
-                            'action': action.rec_name,
-                            })
+                    raise ContextError(
+                        gettext('ir.msg_action_invalid_context',
+                            context=action.context,
+                            action=action.rec_name))
                 else:
                     try:
                         fields.context_validate(value)
                     except Exception:
-                        cls.raise_user_error('invalid_context', {
-                                'context': action.context,
-                                'action': action.rec_name,
-                                })
+                        raise ContextError(
+                            gettext('ir.msg_action_invalid_context',
+                                context=action.context,
+                                action=action.rec_name))
 
     def get_views(self, name):
         return [(view.view.id, view.view.type)
@@ -836,6 +878,8 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
 
     @classmethod
     def get_pyson(cls, windows, name):
+        pool = Pool()
+        encoder = PYSONEncoder()
         pysons = {}
         field = name[6:]
         defaults = {
@@ -844,6 +888,12 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
             'search_value': '[]',
             }
         for window in windows:
+            if not window.order and field == 'order':
+                if window.res_model:
+                    defaults['order'] = encoder.encode(
+                        getattr(pool.get(window.res_model), '_order', 'null'))
+                else:
+                    defaults['order'] = 'null'
             pysons[window.id] = (getattr(window, field)
                 or defaults.get(field, 'null'))
         return pysons
@@ -858,22 +908,26 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
             action_id = ModelData.get_id(*xml_id.split('.'))
         else:
             action_id = int(xml_id)
-        return Action.get_action_values(cls.__name__, [action_id])[0]
+        return Action(action_id).get_action_value()
 
 
-class ActionActWindowView(DeactivableMixin, ModelSQL, ModelView):
+class ActionActWindowView(
+        sequence_ordered(), DeactivableMixin, ModelSQL, ModelView):
     "Action act window view"
     __name__ = 'ir.action.act_window.view'
-    sequence = fields.Integer('Sequence', required=True)
     view = fields.Many2One('ir.ui.view', 'View', required=True,
             ondelete='CASCADE')
     act_window = fields.Many2One('ir.action.act_window', 'Action',
             ondelete='CASCADE')
 
     @classmethod
-    def __setup__(cls):
-        super(ActionActWindowView, cls).__setup__()
-        cls._order.insert(0, ('sequence', 'ASC'))
+    def __register__(cls, module_name):
+        super().__register__(module_name)
+
+        table = cls.__table_handler__(module_name)
+
+        # Migration from 5.0: remove required on sequence
+        table.not_null_action('sequence', 'remove')
 
     @classmethod
     def create(cls, vlist):
@@ -895,24 +949,24 @@ class ActionActWindowView(DeactivableMixin, ModelSQL, ModelView):
         pool.get('ir.action.keyword')._get_keyword_cache.clear()
 
 
-class ActionActWindowDomain(DeactivableMixin, ModelSQL, ModelView):
+class ActionActWindowDomain(
+        sequence_ordered(), DeactivableMixin, ModelSQL, ModelView):
     "Action act window domain"
     __name__ = 'ir.action.act_window.domain'
     name = fields.Char('Name', translate=True)
-    sequence = fields.Integer('Sequence', required=True)
     domain = fields.Char('Domain')
     count = fields.Boolean('Count')
     act_window = fields.Many2One('ir.action.act_window', 'Action',
         select=True, required=True, ondelete='CASCADE')
 
     @classmethod
-    def __setup__(cls):
-        super(ActionActWindowDomain, cls).__setup__()
-        cls._order.insert(0, ('sequence', 'ASC'))
-        cls._error_messages.update({
-                'invalid_domain': ('Invalid domain or search criteria '
-                    '"%(domain)s" on action "%(action)s".'),
-                })
+    def __register__(cls, module_name):
+        super().__register__(module_name)
+
+        table = cls.__table_handler__(module_name)
+
+        # Migration from 5.0: remove required on sequence
+        table.not_null_action('sequence', 'remove')
 
     @classmethod
     def default_count(cls):
@@ -943,10 +997,10 @@ class ActionActWindowDomain(DeactivableMixin, ModelSQL, ModelView):
                 except Exception:
                     value = None
             if value is None:
-                cls.raise_user_error('invalid_domain', {
-                        'domain': action.domain,
-                        'action': action.rec_name,
-                        })
+                raise DomainError(gettext(
+                        'ir.msg_action_invalid_domain',
+                        domain=action.domain,
+                        action=action.rec_name))
 
     @classmethod
     def create(cls, vlist):
@@ -976,13 +1030,23 @@ class ActionWizard(ActionMixin, ModelSQL, ModelView):
     action = fields.Many2One('ir.action', 'Action', required=True,
             ondelete='CASCADE')
     model = fields.Char('Model')
-    email = fields.Char('Email')
-    window = fields.Boolean('Window', help='Run wizard in a new window')
+    window = fields.Boolean('Window', help='Run wizard in a new window.')
 
     @staticmethod
     def default_type():
         return 'ir.action.wizard'
 
+    @classmethod
+    def get_models(cls, name, action_id=None):
+        # TODO add cache
+        domain = [
+            (cls._action_name, '=', name),
+            ]
+        if action_id:
+            domain.append(('id', '=', action_id))
+        actions = cls.search(domain)
+        return {a.model for a in actions if a.model}
+
 
 class ActionURL(ActionMixin, ModelSQL, ModelView):
     "Action URL"
