@@ -4,12 +4,14 @@
 import http.client
 import logging
 import pydoc
+import time
 try:
     from http import HTTPStatus
 except ImportError:
     from http import client as HTTPStatus
 
 from werkzeug.exceptions import abort
+from werkzeug.wrappers import Response
 from sql import Table
 
 from trytond import security
@@ -47,11 +49,9 @@ def rpc(request, database_name):
 
 
 def login(request, database_name, user, parameters, language=None):
-    Database = backend.get('Database')
-    DatabaseOperationalError = backend.get('DatabaseOperationalError')
     try:
-        Database(database_name).connect()
-    except DatabaseOperationalError:
+        backend.Database(database_name).connect()
+    except backend.DatabaseOperationalError:
         logger.error('fail to connect to %s', database_name, exc_info=True)
         abort(HTTPStatus.NOT_FOUND)
     context = {
@@ -87,10 +87,14 @@ def root(request, *args):
     return methods[request.rpc_method](request, *request.rpc_params)
 
 
+@app.route('/<path:path>', methods=['OPTIONS'])
+def options(request, path):
+    return Response(status=HTTPStatus.NO_CONTENT)
+
+
 def db_exist(request, database_name):
-    Database = backend.get('Database')
     try:
-        Database(database_name).connect()
+        backend.Database(database_name).connect()
         return True
     except Exception:
         return False
@@ -102,7 +106,7 @@ def db_list(request, *args):
     context = {'_request': request.context}
     hostname = get_hostname(request.host)
     with Transaction().start(
-            None, 0, context=context, close=True, _nocache=True
+            None, 0, context=context, readonly=True, close=True,
             ) as transaction:
         return transaction.database.list(hostname=hostname)
 
@@ -136,8 +140,6 @@ def help_method(request, pool):
 @app.auth_required
 @with_pool
 def _dispatch(request, pool, *args, **kwargs):
-    DatabaseOperationalError = backend.get('DatabaseOperationalError')
-
     obj, method = get_object_method(request, pool)
     if method in obj.__rpc__:
         rpc = obj.__rpc__[method]
@@ -155,15 +157,18 @@ def _dispatch(request, pool, *args, **kwargs):
                 pool.database_name, user, session, context=context):
             abort(http.client.UNAUTHORIZED)
 
-    log_message = '%s.%s(*%s, **%s) from %s@%s/%s'
+    log_message = '%s.%s(*%s, **%s) from %s@%s%s'
     username = request.authorization.username
     if isinstance(username, bytes):
         username = username.decode('utf-8')
     log_args = (
         obj, method, args, kwargs, username, request.remote_addr, request.path)
-    logger.info(log_message, *log_args)
+    logger.debug(log_message, *log_args)
 
-    for count in range(config.getint('database', 'retry'), -1, -1):
+    retry = config.getint('database', 'retry')
+    for count in range(retry, -1, -1):
+        if count != retry:
+            time.sleep(0.02 * (retry - count))
         with Transaction().start(pool.database_name, user,
                 readonly=rpc.readonly) as transaction:
             try:
@@ -182,7 +187,7 @@ def _dispatch(request, pool, *args, **kwargs):
                     else:
                         result = [rpc.result(meth(i, *c_args, **c_kwargs))
                             for i in inst]
-            except DatabaseOperationalError:
+            except backend.DatabaseOperationalError:
                 if count and not rpc.readonly:
                     transaction.rollback()
                     continue
@@ -204,4 +209,7 @@ def _dispatch(request, pool, *args, **kwargs):
             context = {'_request': request.context}
             security.reset(pool.database_name, session, context=context)
         logger.debug('Result: %s', result)
-        return result
+        response = app.make_response(request, result)
+        if rpc.readonly and rpc.cache:
+            response.headers.extend(rpc.cache.headers())
+        return response
